<?php
/**
 * DBクラス
 *
 * PHP versions 5
 *
 * LICENSE: This source file is licensed under the terms of the GNU General Public License.
 *
 * @package    Magic3 Framework
 * @author     平田直毅(Naoki Hirata) <naoki@aplo.co.jp>
 * @copyright  Copyright 2006-2008 Magic3 Project.
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL License
 * @version    SVN: $Id$
 * @link       http://www.magic3.org
 */
require_once(M3_SYSTEM_INCLUDE_PATH . '/db/baseDb.php');

class SystemDb extends BaseDb
{
	const MAX_ACCESS_LOG_FIELD_LENGTH = 300;				// ログテーブルのフィールドの最大長
	const MAX_ACCESS_LOG_REQUEST_LENGTH = 500;				// リクエストの最大長
	const ADMIN_KEY_LIFETIME = 1440;							// 管理者一時キーの生存期間(分)
	
	/**
	 * システム定義値を取得
	 *
	 * @param string $key    キーとなる項目値
	 * @return string		値
	 */
	function getSystemConfig($key)
	{
		$retValue = '';
		$queryStr = 'SELECT sc_value FROM _system_config ';
		$queryStr .=  'WHERE sc_id  = ?';
		$ret = $this->selectRecord($queryStr, array($key), $row);
		if ($ret) $retValue = $row['sc_value'];
		return $retValue;
	}
	/**
	 * システム定義値を更新
	 *
	 * @param string $key	キーとなる項目値
	 * @param string $value	設定値
	 * @return bool			true=更新成功、false=更新失敗
	 */
	function updateSystemConfig($key, $value)
	{
		$queryStr = "UPDATE _system_config SET sc_value = ? WHERE sc_id = ?";
		$params = array($value, $key);
		$ret = $this->execStatement($queryStr, $params);
		return $ret;
	}
	/**
	 * デザイン定義値を取得
	 *
	 * @param string $key    キーとなる項目値
	 * @return string		値
	 */
	function getDesignConfig($key)
	{
		$retValue = '';
		$queryStr = 'SELECT dn_value FROM _design ';
		$queryStr .=  'WHERE dn_id  = ?';
		$ret = $this->selectRecord($queryStr, array($key), $row);
		if ($ret) $retValue = $row['dn_value'];
		return $retValue;
	}
	/**
	 * デザイン定義値を更新
	 *
	 * @param string $key	キーとなる項目値
	 * @param string $value	設定値
	 * @return				なし
	 */
	function updateDesignConfig($key, $value)
	{
		$queryStr = "UPDATE _design SET dn_value = ? WHERE dn_id = ?";
		$params = array($value, $key);
		$this->execStatement($queryStr, $params);
	}
	/**
	 * アクセスログの記録
	 *
	 * @param int    $user			ログインユーザID(0=不明のとき)
	 * @param string $cookieVal    	アクセス管理用クッキー値
	 * @param string $ip			クライアントIPアドレス
	 * @param string $method		アクセスメソッド
	 * @param string $uri			アクセスURI
	 * @param string $referer		リファラー
	 * @param string $request		リクエストパラメータ
	 * @param string $agent			クライアントアプリケーション
	 * @param string $language		クライアント認識可能言語
	 * @return int					アクセスログシリアルNo(ログ保存失敗のときは0)
	 */
	function accessLog($user, $cookieVal, $ip, $method, $uri, $referer, $request, $agent, $language)
	{
		// 格納長を制限
		if (function_exists('mb_substr')){
			$uri = mb_substr($uri, 0, self::MAX_ACCESS_LOG_FIELD_LENGTH);	
			$referer = mb_substr($referer, 0, self::MAX_ACCESS_LOG_FIELD_LENGTH);
			$agent = mb_substr($agent, 0, self::MAX_ACCESS_LOG_FIELD_LENGTH);
			$request = mb_substr($request, 0, self::MAX_ACCESS_LOG_REQUEST_LENGTH);
		} else {
			$uri = substr($uri, 0, self::MAX_ACCESS_LOG_FIELD_LENGTH);	
			$referer = substr($referer, 0, self::MAX_ACCESS_LOG_FIELD_LENGTH);
			$agent = substr($agent, 0, self::MAX_ACCESS_LOG_FIELD_LENGTH);
			$request = substr($request, 0, self::MAX_ACCESS_LOG_REQUEST_LENGTH);
		}
		// トランザクション開始
		$this->startTransaction();
		
		$queryStr = 'INSERT INTO _access_log (';
		$queryStr .=  'al_user_id, ';
		$queryStr .=  'al_cookie_value, ';
		$queryStr .=  'al_ip, ';
		$queryStr .=  'al_method, ';
		$queryStr .=  'al_uri, ';
		$queryStr .=  'al_referer, ';
		$queryStr .=  'al_request, ';
		$queryStr .=  'al_user_agent, ';
		$queryStr .=  'al_accept_language, ';
		$queryStr .=  'al_dt ';
		$queryStr .=  ') VALUES (';
		$queryStr .=  '?, ?, ?, ?, ?, ?, ?, ?, ?, now()';
		$queryStr .=  ')';
		$ret = $this->execStatement($queryStr, array($user, $cookieVal, $ip, $method, $uri, $referer, $request, $agent, $language));
		
		$serial = $this->getMaxSerialOfAccessLog();
		
		// トランザクション終了
		$ret = $this->endTransaction();
		if ($ret){
			return $serial;
		} else {
			return 0;
		}
	}
	/**
	 * アクセスログのユーザIDを更新
	 *
	 * @param int    $serialNo		ログシリアルNo
	 * @param int    $userid		ユーザID
	 * @return bool					true=更新成功、false=更新失敗
	 */
	function updateAccessLogUser($serialNo, $userid)
	{
		// トランザクション開始
		$this->startTransaction();
		
		$sql = "UPDATE _access_log SET al_user_id = ? WHERE al_serial = ?";
		$params = array($userid, $serialNo);
		$this->execStatement($sql, $params);
			
		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * アクセスログの最大シリアルNoを取得
	 *
	 * @return int		最大シリアルNo
	 */
	function getMaxSerialOfAccessLog()
	{
		$retValue = 0;
		$queryStr = 'select max(al_serial) as m from _access_log';
		$ret = $this->selectRecord($queryStr, array(), $row);
		if ($ret) $retValue = $row['m'];
		return $retValue;
	}
	/**
	 * デフォルト言語を取得
	 *
	 * @param string	デフォルト言語
	 */
	function getDefaultLanguage()
	{
		$retValue = '';
		$queryStr = 'select sc_value from _system_config ';
		$queryStr .=  'where sc_id = ?';
		$ret = $this->selectRecord($queryStr, array('default_lang'), $row);
		if ($ret) $retValue = $row['sc_value'];
		return $retValue;
	}
	/**
	 * 表示言語変更可能かどうかを取得
	 */
	function getCanChangeLang()
	{
		$retValue = false;
		$queryStr = 'select sc_value from _system_config ';
		$queryStr .=  'where sc_id = ?';
		$ret = $this->selectRecord($queryStr, array('permit_change_lang'), $row);
		if ($ret){
			if ($row['sc_value'] == '1') $retValue = true;
		}
		return $retValue;
	}
	/**
	 * 言語名称を取得
	 *
	 * @param string $langId			言語ID
	 * @param string $dispLangId    	表示言語ID
	 * @return string					言語名称
	 */
	function getLanguageNameByDispLanguageId($langId, $dispLangId)
	{
		$retValue = '';
		$queryStr = 'SELECT * from _language ';
		$queryStr .=  'WHERE ln_id = ?';
		$ret = $this->selectRecord($queryStr, array($langId), $row);
		if ($ret){
			if ($dispLangId == 'ja'){	// 日本語のとき
				$retValue = $row['ln_name'];
			} else {
				$retValue = $row['ln_name_en'];
			}
		}
		return $retValue;
	}
	/**
	 * デフォルトのページサブIDを取得
	 *
	 * @param string $pageId	ページID
	 * @return string			デフォルトのテンプレートID
	 */
	function getDefaultPageSubId($pageId)
	{
		$retValue = '';
		$queryStr = 'SELECT pg_default_sub_id FROM _page_id ';
		$queryStr .=  'WHERE pg_id = ? AND pg_type = 0';
		$ret = $this->selectRecord($queryStr, array($pageId), $row);
		if ($ret) $retValue = $row['pg_default_sub_id'];
		return $retValue;
	}
	/**
	 * 指定のページがアクセス可能なページかチェック
	 *
	 * @param string $pageId		ページID
	 * @param string $pageSubId		ページサブID
	 * @return bool					true=可能、false=不可
	 */
	function canAccessPage($pageId, $pageSubId)
	{
		// ページIDのアクセスチェック
		$queryStr = 'SELECT * FROM _page_id ';
		$queryStr .=  'WHERE pg_type = 0 AND pg_id = ?';
		$ret = $this->selectRecord($queryStr, array($pageId), $row);
		if ($ret){
			if (!$row['pg_is_public']) return false;
		} else {
			return false;
		}

		// ページサブIDのアクセスチェック
		$queryStr = 'SELECT * FROM _page_id ';
		$queryStr .=  'WHERE pg_type = 1 AND pg_id = ?';
		$ret = $this->selectRecord($queryStr, array($pageSubId), $row);
		if ($ret){
			if (!$row['pg_is_public']) return false;
		} else {
			return false;
		}
		return true;
	}
	/**
	 * 指定のウィジェットがアクセス可能なウィジェットかチェック
	 *
	 * アクセス可能なウィジェットは以下のすべての条件を満たすもの
	 * ・ウィジェット単体起動が許可されている
	 * ・ウィジェットが公開ページ上にある
	 *
	 * @param string $widgetId		ウィジェットID
	 * @param int    $setId			定義セットID
	 * @return bool					true=可能、false=不可
	 */
	function canAccessWidget($widgetId, $setId = 0)
	{
		$queryStr  = 'SELECT * ';
		$queryStr .= 'FROM (_page_def LEFT JOIN _widgets ON pd_widget_id = wd_id AND wd_deleted = false) ';
		$queryStr .=   'LEFT JOIN _page_id ON pd_sub_id = pg_id AND pg_type = 1 ';// ページサブID
		$queryStr .= 'WHERE pd_widget_id = ? ';
		$queryStr .=   'AND pd_set_id = ? ';
		$queryStr .=   'AND pd_visible = true ';			// ウィジェットは表示中
		$queryStr .=   'AND wd_deleted = false ';			// ウィジェットは削除されていない
		$queryStr .=   'AND wd_enable_operation = true ';	// 単体実行可能
		$queryStr .=   'AND pg_is_public = true ';			// 公開中のページ
		$params = array($widgetId, $setId);
		$ret = $this->isRecordExists($queryStr, $params);
		return $ret;
	}
	/**
	 * デフォルトのテンプレートIDを取得
	 *
	 * @return string	デフォルトのテンプレートID
	 */
	function getDefaultTemplateId()
	{
		$retValue = '';
		$queryStr = 'SELECT tm_id FROM _templates ';
		$queryStr .=  'WHERE tm_default = true ';
		$queryStr .=  'AND tm_mobile = false ';		// 携帯用でない
		$queryStr .=  'AND tm_deleted = false';
		$ret = $this->selectRecord($queryStr, array(), $row);
		if ($ret) $retValue = $row['tm_id'];
		return $retValue;
	}
	/**
	 * 携帯用デフォルトのテンプレートIDを取得
	 *
	 * @return string	デフォルトのテンプレートID
	 */
	function getDefaultMobileTemplateId()
	{
		$retValue = '';
		$queryStr = 'SELECT tm_id FROM _templates ';
		$queryStr .=  'WHERE tm_default = true ';
		$queryStr .=  'AND tm_mobile = true ';		// 携帯用
		$queryStr .=  'AND tm_deleted = false';
		$ret = $this->selectRecord($queryStr, array(), $row);
		if ($ret) $retValue = $row['tm_id'];
		return $retValue;
	}
	/**
	 * テンプレート情報の取得
	 *
	 * @param string  $id			テンプレートID
	 * @return						true=正常、false=異常
	 */
	function getTemplate($id, &$row)
	{
		$queryStr  = 'SELECT * FROM _templates ';
		$queryStr .=   'WHERE tm_id = ? ';
		$queryStr .=   'AND tm_deleted = false';
		$ret = $this->selectRecord($queryStr, array($id), $row);
		return $ret;
	}
	/**
	 * 画面情報を取得
	 *
	 * @param string $pageId		ページID
	 * @param string $subpage    	ページサブID
	 * @param string $position		表示位置ID
	 * @param array  $rows			レコード
	 * @param int    $setId			定義セットID
	 * @return bool					1行以上取得 = true, 取得なし= false
	 */
	function getPageDef($pageId, $pageSubId, $position, &$rows, $setId = 0)
	{
		$queryStr  = 'select * from _page_def left join _widgets on pd_widget_id = wd_id and wd_deleted = false ';
		$queryStr .=   'where pd_visible = true ';		// ウィジェットを表示
		$queryStr .=     'and pd_id = ? ';
		$queryStr .=     'and (pd_sub_id = ? or pd_sub_id = \'\') ';	// 空の場合は共通項目
		$queryStr .=     'and pd_position_id = ? ';
		$queryStr .=     'and pd_set_id = ? ';
		$queryStr .=   'order by pd_position_id, pd_index';
		$retValue = $this->selectRecords($queryStr, array($pageId, $pageSubId, $position, $setId), $rows);
		return $retValue;
	}
	/**
	 * ウィジェットが指定のページID上にあるかどうか判断
	 *
	 * @param string $pageId			ページID
	 * @param string $subpage    		ページサブID
	 * @param string $widgetId			ウィジェットID
	 * @param int    $setId				定義セットID
	 * @return bool						true=ページ上にあり、false=ページ上になし
	 */
	function isExistsWidgetOnPage($pageId, $pageSubId, $widgetId, $setId = 0)
	{
		$queryStr  = 'SELECT * FROM _page_def LEFT JOIN _widgets ON pd_widget_id = wd_id AND wd_deleted = false ';
		$queryStr .=   'WHERE pd_id = ? ';
		$queryStr .=     'AND (pd_sub_id = ? OR pd_sub_id = \'\') ';	// 空の場合は共通項目
		$queryStr .=     'AND pd_widget_id = ? ';
		$queryStr .=     'AND pd_set_id = ? ';
		$params = array($pageId, $pageSubId, $widgetId, $setId);
		$ret = $this->isRecordExists($queryStr, $params);
		return $ret;
	}
	/**
	 * 指定したウィジェットの表示できるサブページIDを取得
	 *
	 * @param string $widgetId		ウィジェットID
	 * @param string $pageId		ページID
	 * @param array  $rows			レコード
	 * @param int    $setId			定義セットID
	 * @return bool					1行以上取得 = true, 取得なし= false
	 */
	function getSubPageId($widgetId, $pageId, &$rows, $setId = 0)
	{	
		$queryStr  = 'SELECT DISTINCT pd_sub_id, ';
		$queryStr .=   'CASE pd_sub_id ';
		$queryStr .=     'WHEN \'\' THEN -1 ';
		$queryStr .=     'ELSE pg_priority ';
		$queryStr .=   'END AS idx ';
		$queryStr .= 'FROM (_page_def LEFT JOIN _widgets ON pd_widget_id = wd_id AND wd_deleted = false) ';
		$queryStr .=   'LEFT JOIN _page_id ON pd_sub_id = pg_id AND pg_type = 1 ';// ページサブID
		$queryStr .= 'WHERE pd_id = ? ';
		$queryStr .=   'AND pd_widget_id = ? ';
		$queryStr .=   'AND pd_set_id = ? ';
		$queryStr .=   'AND pg_is_public = true ';			// 公開中のページ
		$queryStr .= 'ORDER BY idx';
		$retValue = $this->selectRecords($queryStr, array($pageId, $widgetId, $setId), $rows);
		return $retValue;
	}
	/**
	 * 互換性のあるウィジェットのIDを取得
	 *
	 * @param string $widgetId	ウィジェットID
	 * @return string 			互換ウィジェットID(見つからないときは空文字列を返す)
	 */
	function getCompatibleWidgetId($widgetId)
	{
		$retValue = '';
		$queryStr = 'SELECT * FROM _widgets ';
		$queryStr .=  'WHERE wd_deleted = false ';// 削除されていない
		$queryStr .=  'AND wd_compatible_id = ? ';
		$queryStr .=  'ORDER BY wd_id DESC';
		$ret = $this->selectRecord($queryStr, array($widgetId), $row);
		if ($ret) $retValue = $row['wd_id'];
		return $retValue;
	}
	/**
	 * サイト定義を取得
	 *
	 * @param string $key    キーとなる項目値
	 * @param string $lang   言語ID
	 */
	function getSiteDefValue($key, $lang = '')
	{
		global $gEnvManager;
		
		if (empty($lang)){	// 空の場合はデフォルト言語ID取得
			$language = $gEnvManager->getDefaultLanguage();
		} else {
			$language = $lang;
		}
		$retValue = '';
		$queryStr = 'select sd_value from _site_def ';
		$queryStr .=  'where sd_deleted = false and sd_id = ? and sd_language_id = ?';
		$ret = $this->selectRecord($queryStr, array($key, $language), $row);
		if ($ret) $retValue = $row['sd_value'];
		return $retValue;
	}
	/**
	 * スクリプトファイルが必要なウィジェットIDを取得
	 *
	 * @param string $pageid		ページID
	 * @param string $subpageid    	ページサブID
	 * @param array  $rows			レコード
	 * @param int    $setId				定義セットID
	 * @return bool					1行以上取得 = true, 取得なし= false
	 */
	function getWidgetsIdWithScript($pageid, $subpageid, &$rows, $setId = 0)
	{
		$queryStr = 'select distinct wd_id from _page_def join _widgets on pd_widget_id = wd_id and wd_deleted = false ';
		$queryStr .=  'where pd_visible = true ';
		$queryStr .=    'and pd_id = ? ';
		//$queryStr .=    'and pd_sub_id = ? ';
		$queryStr .=     'and (pd_sub_id = ? or pd_sub_id = \'\') ';	// 空の場合は共通項目
		$queryStr .=    'and pd_set_id = ? ';
		$queryStr .=    'and wd_read_scripts = true ';
		$queryStr .=  'order by wd_id';
		
		$retValue = $this->selectRecords($queryStr, array($pageid, $subpageid, $setId), $rows);
		return $retValue;
	}
	/**
	 * CSSファイルが必要なウィジェットIDを取得
	 *
	 * @param string $pageid		ページID
	 * @param string $subpageid    	ページサブID
	 * @param array  $rows			レコード
	 * @param int    $setId			定義セットID
	 * @return bool					1行以上取得 = true, 取得なし= false
	 */
	function getWidgetsIdWithCss($pageid, $subpageid, &$rows, $setId = 0)
	{
		$queryStr = 'select distinct wd_id from _page_def join _widgets on pd_widget_id = wd_id and wd_deleted = false ';
		$queryStr .=  'where pd_visible = true ';
		$queryStr .=    'and pd_id = ? ';
		//$queryStr .=    'and pd_sub_id = ? ';
		$queryStr .=     'and (pd_sub_id = ? or pd_sub_id = \'\') ';	// 空の場合は共通項目
		$queryStr .=    'and pd_set_id = ? ';
		$queryStr .=    'and wd_read_css = true ';
		$queryStr .=  'order by wd_id';
		
		$retValue = $this->selectRecords($queryStr, array($pageid, $subpageid, $setId), $rows);
		return $retValue;
	}
	/**
	 * 共通Ajaxライブラリを使用しているウィジェットがあるかどうか
	 *
	 * @param string $pageid		ページID
	 * @param string $subpageid    	ページサブID
	 * @param array  $rows			レコード
	 * @param int    $setId			定義セットID
	 * @return bool					1行以上取得 = true, 取得なし= false
	 */
	function isExistsWidgetWithAjax($pageid, $subpageid, $setId = 0)
	{
		$queryStr = 'select distinct wd_id from _page_def join _widgets on pd_widget_id = wd_id and wd_deleted = false ';
		$queryStr .=  'where pd_visible = true ';
		$queryStr .=    'and pd_id = ? ';
		$queryStr .=     'and (pd_sub_id = ? or pd_sub_id = \'\') ';	// 空の場合は共通項目
		$queryStr .=    'and pd_set_id = ? ';
		$queryStr .=    'and wd_use_ajax = true ';		// Ajaxを使用
		$queryStr .=  'order by wd_id';
		return $this->isRecordExists($queryStr, array($pageid, $subpageid, $setId));
	}
	/**
	 * ウィジェット情報取得
	 *
	 * @param string $widgetId		ウィジェットID
	 * @param array $row			取得した行
	 * @return bool					取得 = true, 取得なし= false
	 */
	function getWidgetInfo($widgetId, &$row)
	{
		$queryStr = 'SELECT * from _widgets ';
		$queryStr .=  'WHERE wd_deleted = false ';// 削除されていない
		$queryStr .=  'AND wd_id = ?';
		return $this->selectRecord($queryStr, array($widgetId), $row);
	}
	/**
	 * ウィジェットの定義IDを更新する
	 *
	 * @param string $widgetId		ウィジェットID
	 * @param int    $serial		シリアル番号
	 * @param int    $configId		定義ID
	 * @param int    $title			定義名
	 * @return bool				true=成功、false=失敗
	 */
	function updateWidgetConfigId($widgetId, $serial, $configId, $title = '')
	{
		global $gEnvManager;

		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
		
		// トランザクション開始
		$this->startTransaction();

		// 現在の値取得
		$queryStr  = 'SELECT * FROM _page_def ';
		$queryStr .=   'WHERE pd_serial = ? ';
		$ret = $this->selectRecord($queryStr, array($serial), $row);
		if (!$ret){		// 登録レコードがないとき
			$this->endTransaction();
			return false;
		}
		if ($row['pd_widget_id'] != $widgetId){		// ウィジェットIDが異なるとき
			$this->endTransaction();
			return false;
		}
		// 既存項目を更新
		$queryStr  = 'UPDATE _page_def ';
		$queryStr .=   'SET ';
		$queryStr .=     'pd_config_id = ?, ';
		$queryStr .=     'pd_config_name = ?, ';
		$queryStr .=     'pd_update_user_id = ?, ';
		$queryStr .=     'pd_update_dt = ? ';
		$queryStr .=   'WHERE pd_serial = ? ';
		$this->execStatement($queryStr, array($configId, $title, $userId, $now, $serial));
			
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ウィジェット表示順を変更する
	 *
	 * @param string $pageId		ページID
	 * @param string $pageSubId		ページサブID
	 * @param string $position		ポジション
	 * @param array $widgetNoArray	ウィジェット並び順
	 * @param int    $setId				定義セットID
	 * @return bool				true=成功、false=失敗
	 */
	function orderWidget($pageId, $pageSubId, $position, $widgetArray, $setId = 0)
	{
		global $gEnvManager;

		// 指定ポジションのウィジェットをすべて取得
		$ret = $this->getPageDef($pageId, $pageSubId, $position, $rows, $setId);
		if (!$ret) return false;
	
		// ウィジェット数をチェックし、異なっている場合はエラー
		$widgetCount = count($rows);
		if ($widgetCount != count($widgetArray)) return false;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
		
		// トランザクション開始
		$this->startTransaction();

		for ($i = 0; $i < $widgetCount; $i++){
			$serialNo = $rows[$widgetArray[$i]]['pd_serial'];
			$index = $rows[$i]['pd_index'];
			
			// 既存項目を更新
			$queryStr  = 'UPDATE _page_def ';
			$queryStr .=   'SET ';
			$queryStr .=     'pd_index = ?, ';
			$queryStr .=     'pd_update_user_id = ?, ';
			$queryStr .=     'pd_update_dt = ? ';
			$queryStr .=   'WHERE pd_serial = ? ';
			$this->execStatement($queryStr, array($index, $userId, $now, $serialNo));
		}
										
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ウィジェットを指定ポジションに追加する
	 *
	 * @param string $pageId		ページID
	 * @param string $pageSubId		ページサブID
	 * @param string $position		ポジション
	 * @param string $widgetId		追加ウィジェットID
	 * @param int $posIndex			ポジション内の位置
	 * @param int    $setId			定義セットID
	 * @return bool				true=成功、false=失敗
	 */
	function addWidget($pageId, $pageSubId, $position, $widgetId, $posIndex, $setId = 0)
	{
		global $gEnvManager;

//echo $pageId . '-' . $pageSubId . '-' . $position . '-' . $widgetId . '-' . $posIndex;
		// 指定ポジションのウィジェットをすべて取得
		$ret = $this->getPageDef($pageId, $pageSubId, $position, $rows, $setId);
		if (!$ret) $rows = array();
	
		// 指定インデックスのエラーチェック
		$widgetCount = count($rows);
		if ($posIndex < 0 || $widgetCount < $posIndex) return false;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
		
		// トランザクション開始
		$this->startTransaction();

		// 新規追加ウィジェットの表示順を決定
		if ($widgetCount == 0){		// ポジションにウィジェットがないとき
			$newIndex = 5;
		} else {
			if ($posIndex == 0){		// すべてのウィジェットよりも先頭のとき
				$newIndex = $rows[$posIndex]['pd_index'] - 5;
				if ($newIndex < 1){			// 表示順振りなおし
					for ($i = 0; $i < $widgetCount; $i++){
						$serialNo = $rows[$i]['pd_serial'];
						$index = $i * 5 + 15;			// 5とばしで設定
						
						// 既存項目を更新
						$queryStr  = 'UPDATE _page_def ';
						$queryStr .=   'SET ';
						$queryStr .=     'pd_index = ?, ';
						$queryStr .=     'pd_update_user_id = ?, ';
						$queryStr .=     'pd_update_dt = ? ';
						$queryStr .=   'WHERE pd_serial = ? ';
						$this->execStatement($queryStr, array($index, $userId, $now, $serialNo));
					}
					$newIndex = 10;
				}
			} else if ($posIndex == $widgetCount){// すべてのウィジェットの最後のとき
				$newIndex = $rows[$posIndex -1]['pd_index'] + 5;
			} else {
				$foreIndex = $rows[$posIndex -1]['pd_index'];
				$nextIndex = $rows[$posIndex]['pd_index'];
				
				// 間隔がつまっているときは、表示順を振りなおす
				if ($nextIndex - $foreIndex > 1){
					$newIndex = $foreIndex + intval(($nextIndex - $foreIndex) / 2);
				} else {
					for ($i = 0; $i < $widgetCount; $i++){
						$serialNo = $rows[$i]['pd_serial'];
						
						if ($i < $posIndex){
							$index = $i * 5 + 10;			// 5とばしで設定
						} else {
							$index = $i * 5 + 15;			// 5とばしで設定
						}
						
						// 既存項目を更新
						$queryStr  = 'UPDATE _page_def ';
						$queryStr .=   'SET ';
						$queryStr .=     'pd_index = ?, ';
						$queryStr .=     'pd_update_user_id = ?, ';
						$queryStr .=     'pd_update_dt = ? ';
						$queryStr .=   'WHERE pd_serial = ? ';
						$this->execStatement($queryStr, array($index, $userId, $now, $serialNo));
					}
					$newIndex = $posIndex * 5 + 10;
				}
			}
		}
		// 新規データを追加
		$configId = 0;
		$suffix = '';
		$style = '';
		$visible = 1;
		$editable = 1;
		$queryStr  = 'INSERT INTO _page_def (';
		$queryStr .=   'pd_id, ';
		$queryStr .=   'pd_sub_id, ';
		$queryStr .=   'pd_set_id, ';
		$queryStr .=   'pd_position_id, ';
		$queryStr .=   'pd_index, ';
		$queryStr .=   'pd_widget_id, ';
		$queryStr .=   'pd_config_id, ';
		$queryStr .=   'pd_suffix, ';
		$queryStr .=   'pd_style, ';
		$queryStr .=   'pd_visible, ';
		$queryStr .=   'pd_editable, ';
		$queryStr .=   'pd_update_user_id, ';
		$queryStr .=   'pd_update_dt) ';
		$queryStr .= 'VALUES (';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?)';
		$this->execStatement($queryStr, array($pageId, $pageSubId, $setId, $position, $newIndex, 
							$widgetId, $configId, $suffix, $style, $visible, $editable, $userId, $now));
								
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ウィジェットを削除する
	 *
	 * @param string $pageId		ページID
	 * @param string $pageSubId		ページサブID
	 * @param string $position		ポジション
	 * @param string $widgetId		ウィジェットID
	 * @param int $index			ポジション内の位置
	 * @param int $widCount;		ポジション内のウィジェット数
	 * @param int    $setId				定義セットID
	 * @return bool				true=成功、false=失敗
	 */
	function delWidget($pageId, $pageSubId, $position, $widget, $index, $widCount, $setId = 0)
	{
		global $gEnvManager;

		// 指定ポジションのウィジェットをすべて取得
		$ret = $this->getPageDef($pageId, $pageSubId, $position, $rows, $setId);
		if (!$ret) return true;
	
		// 指定インデックスのエラーチェック
		$widgetCount = count($rows);
		if ($widgetCount != $widCount) return false;
		if ($posIndex < 0 || $widgetCount <= $posIndex) return false;
		
		$serialNo = $rows[$index]['pd_serial'];
		$widgetId = $rows[$index]['pd_widget_id'];
		if ($widgetId != $widget) return false;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
		
		// トランザクション開始
		$this->startTransaction();
		
		$queryStr  = 'DELETE FROM _page_def WHERE pd_serial = ?';
		$this->execStatement($queryStr, array($serialNo));
		
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ウィジェットの共通属性を変更
	 *
	 * @param string $pageId		ページID
	 * @param string $pageSubId		ページサブID
	 * @param string $position		ポジション
	 * @param string $widgetId		ウィジェットID
	 * @param int $index			ポジション内の位置
	 * @param int $widCount;		ポジション内のウィジェット数
	 * @param int    $commonStatus		現在の共通属性(0=単独、1=共通)
	 * @param int    $setId				定義セットID
	 * @return bool				true=成功、false=失敗
	 */
	function toggleCommonWidget($pageId, $pageSubId, $position, $widget, $index, $widCount, $commonStatus, $setId = 0)
	{
		global $gEnvManager;

		// 指定ポジションのウィジェットをすべて取得
		$ret = $this->getPageDef($pageId, $pageSubId, $position, $rows, $setId);
		if (!$ret) return true;
	
		// 指定インデックスのエラーチェック
		$widgetCount = count($rows);
		if ($widgetCount != $widCount) return false;
		if ($posIndex < 0 || $widgetCount <= $posIndex) return false;
		
		$serialNo = $rows[$index]['pd_serial'];		// シリアル番号
		$widgetId = $rows[$index]['pd_widget_id'];
		if ($widgetId != $widget) return false;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
		
		// トランザクション開始
		$this->startTransaction();
		
		// 現在の値取得
		$queryStr  = 'select * from _page_def ';
		$queryStr .=   'where pd_serial = ? ';
		$ret = $this->selectRecord($queryStr, array($serialNo), $row);
		if ($ret){
			if (empty($row['pd_sub_id']) && $commonStatus == 1){		// 共通ウィジェットのとき
				$newPageSubId = $pageSubId;
			} else if (!empty($row['pd_sub_id']) && $commonStatus == 0){		// 単独ウィジェットのとき
				$newPageSubId = '';
			} else {		// エラーのとき
				$this->endTransaction();
				return false;
			}
			// 新規に追加して、古いレコードを削除
			// 新規データを追加
			$queryStr  = 'INSERT INTO _page_def (';
			$queryStr .=   'pd_id, ';
			$queryStr .=   'pd_sub_id, ';
			$queryStr .=   'pd_set_id, ';
			$queryStr .=   'pd_position_id, ';
			$queryStr .=   'pd_index, ';
			$queryStr .=   'pd_widget_id, ';
			$queryStr .=   'pd_config_id, ';
			$queryStr .=   'pd_suffix, ';
			$queryStr .=   'pd_style, ';
			$queryStr .=   'pd_visible, ';
			$queryStr .=   'pd_editable, ';
			$queryStr .=   'pd_update_user_id, ';
			$queryStr .=   'pd_update_dt) ';
			$queryStr .= 'VALUES (';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?, ';
			$queryStr .=   '?)';
			$this->execStatement($queryStr, array($pageId, $newPageSubId, $row['pd_set_id'], $row['pd_position_id'], $row['pd_index'], 
								$row['pd_widget_id'], $row['pd_config_id'], $row['pd_suffix'], $row['pd_style'], $row['pd_visible'], $row['pd_editable'], $userId, $now));
								
			// 旧データ削除
			$queryStr  = 'DELETE FROM _page_def WHERE pd_serial = ?';
			$this->execStatement($queryStr, array($serialNo));
		}
		
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * インナーウィジェットリスト取得
	 *
	 * @param  string $widgetId	ウィジェットID
	 * @param  string $type			インナーウィジェットのタイプ
	 * @param function $callback	コールバック関数
	 * @return						なし
	 */
	function getAllIWidgetListByType($widgetId, $type, $callback)
	{
		$queryStr = 'SELECT * FROM _iwidgets ';
		$queryStr .=  'WHERE iw_deleted = false ';// 削除されていない
		$queryStr .=  'AND (iw_widget_id = ? OR iw_id = \'\') ';	// 空の場合は共通項目
		$queryStr .=  'AND iw_type = ? ';
		$queryStr .=  'ORDER BY iw_id';
		$this->selectLoop($queryStr, array($widgetId, $type), $callback);
	}
	/**
	 * インナーウィジェット情報取得
	 *
	 * @param string $widgetId		ウィジェットID
	 * @param string $id			インナーウィジェットID
	 * @param array $row			取得した行
	 * @return bool					取得 = true, 取得なし= false
	 */
	function getIWidgetInfo($widgetId, $id, &$row)
	{
		$queryStr = 'SELECT * from _iwidgets ';
		$queryStr .=  'WHERE iw_deleted = false ';// 削除されていない
		$queryStr .=  'AND iw_widget_id = ?';
		$queryStr .=  'AND iw_id = ?';
		return $this->selectRecord($queryStr, array($widgetId, $id), $row);
	}
	/**
	 * セッションデータ取得
	 *
	 * @param  string $id	セッションID
	 * @return string		セッションデータ
	 */
	function readSession($id)
	{
		$sql = 'SELECT ss_data FROM _session WHERE ss_id = ?';
		$params = array($id); 
		$this->selectRecord($sql, $params, $row);
		return $row['ss_data'];
    }
	/**
	 * セッションデータ書き込み
	 *
	 * @param  string $id		セッションID
	 * @param  string $sessData	セッションデータ
	 * @return bool				true=成功、false=失敗
	 */
	function writeSession($id, $sessData)
	{
		// トランザクション開始
		$this->startTransaction();
		
		// レコードが存在するときは、更新
        $sql="SELECT ss_id FROM _session WHERE ss_id = ?";
        $params = array($id);
        $ret = $this->isRecordExists($sql, $params);
		if ($ret){
			$sql = "UPDATE _session SET ss_data = ?, ss_update_dt = now() WHERE ss_id = ?";
			$params = array($sessData, $id);
			$this->execStatement($sql, $params);
		} else {
			$sql = "INSERT INTO _session (ss_id, ss_data, ss_update_dt) VALUES (?, ?, now())";
			$params = array($id, $sessData);
			$this->execStatement($sql, $params);
		}
		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * セッションレコード削除
	 *
	 * @param  string $id	セッションID
	 * @return bool			削除の結果
	 */
    function destroySession($id)
	{
		// トランザクション開始
		$this->startTransaction();
		
		$sql = "DELETE FROM _session WHERE ss_id = ?";
		$params = array($id);
		$this->execStatement($sql, $params);
		
		// トランザクション終了
		return $this->endTransaction();
    }
	/**
	 * 一定期間経過しているセッションレコード削除
	 *
	 * @param  int $maxlifetime	削除までの期間
	 * @return bool				削除の結果
	 */
	function gcSession($maxlifetime)
	{
		global $gLogManager;
		$gLogManager->debug(__METHOD__, 'セッションガーベージコレクション開始');// debugログ出力

		// トランザクション開始
		$this->startTransaction();

		if ($this->getDbType() == M3_DB_TYPE_MYSQL){		// MySQLの場合
			$sql = "DELETE FROM _session WHERE UNIX_TIMESTAMP() - UNIX_TIMESTAMP(ss_update_dt) > ?";
			$params = array($maxlifetime);
		} else if ($this->getDbType() == M3_DB_TYPE_PGSQL){// PostgreSQLの場合
			//$sql = "DELETE FROM _session WHERE 'now'::timestamp - ss_update_dt > '? seconds'";
			$sql = "DELETE FROM _session WHERE 'now'::timestamp - ss_update_dt > '" . $maxlifetime . " seconds'";
			$params = array();
		}
		$this->execStatement($sql, $params);
		
		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * ログインユーザ情報取得
	 *
	 * @param  string $account	ユーザアカウント
	 * @param  array  $row		取得レコード
	 * @return bool				true=正常取得、false=取得できず
	 */
	function getLoginUserRecord($account, &$row)
	{
		$queryStr = 'select * from _login_user ';
		$queryStr .=  'where lu_account = ? ';
		$queryStr .=    'and lu_deleted = false';
		$ret = $this->selectRecord($queryStr, array($account), $row);
		return $ret;
	}
	/**
	 * ユーザアカウントが存在するかチェック
	 *
	 * @param string $account	アカウント
	 * @return					true=存在する、false=存在しない
	 */
	function isExistsAccount($account)
	{
		$queryStr = 'SELECT * from _login_user ';
		$queryStr .=  'WHERE lu_account = ? ';
		$queryStr .=    'AND lu_deleted = false';
		return $this->isRecordExists($queryStr, array($account));
	}
	/**
	 * ログインユーザ情報取得をIDで取得
	 *
	 * @param int	$id						ユーザID
	 * @param array     $row				レコード
	 * @return bool							取得 = true, 取得なし= false
	 */
	function getLoginUserRecordById($id, &$row)
	{
		$queryStr  = 'select * from _login_user ';
		$queryStr .=   'WHERE lu_id = ? ';
		$queryStr .=    'and lu_deleted = false';
		$ret = $this->selectRecord($queryStr, array($id), $row);
		return $ret;
	}
	/**
	 * ログインユーザ情報をIDで削除
	 *
	 * @param int	$id						ユーザID
	 * @return bool							取得 = true, 取得なし= false
	 */
	function delLoginUser($id)
	{
		global $gEnvManager;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
		
		// トランザクション開始
		$this->startTransaction();
		
		// 指定のIDのレコードが削除状態でないかチェック
		$queryStr  = 'select * from _login_user ';
		$queryStr .=   'WHERE lu_id = ? ';
		$queryStr .=    'and lu_deleted = false';
		$ret = $this->selectRecord($queryStr, array($id), $row);
		if (!$ret){		// 登録レコードがないとき
			$this->endTransaction();
			return false;
		}
		
		// 古いレコードを削除
		$queryStr  = 'UPDATE _login_user ';
		$queryStr .=   'SET lu_deleted = true, ';	// 削除
		$queryStr .=     'lu_update_user_id = ?, ';
		$queryStr .=     'lu_update_dt = ? ';
		$queryStr .=   'WHERE lu_serial = ?';
		$this->execStatement($queryStr, array($userId, $now, $row['lu_serial']));
		
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * 管理者権限があるかどうかをIDで取得
	 *
	 * @param int	$id						ユーザID
	 * @return bool							管理権限あり=true, 管理権限なし=false
	 */
	function isSystemAdmin($userId)
	{
		$ret = $this->getLoginUserRecordById($userId, $row);
		if ($ret && $row['lu_user_type'] >= UserInfo::USER_TYPE_SYS_ADMIN){		// 管理者権限ありのとき
			return true;
		} else {
			return false;
		}
	}
	/**
	 * ログインユーザのパスワードを更新
	 *
	 * @param int	$id				ユーザID
	 * @param string  $password		パスワード
	 * @param bool  $isMd5			MD5化されたパスワードかどうか
	 * @return						true=成功、false=失敗
	 */
	function updateLoginUserPassword($id, $password, $isMd5=false)
	{
		global $gEnvManager;
		
		$startTran = false;			// この関数でトランザクションを開始したかどうか
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
				
		// トランザクション開始
		if (!$this->isInTransaction()){
			$this->startTransaction();
			$startTran = true;
		}
		
		// 指定のIDのレコードが削除状態でないかチェック
		$queryStr  = 'select * from _login_user ';
		$queryStr .=   'WHERE lu_id = ? ';
		$queryStr .=    'and lu_deleted = false';
		$ret = $this->selectRecord($queryStr, array($id), $row);
		if (!$ret){		// 登録レコードがないとき
			if ($startTran) $this->endTransaction();
			return false;
		}
		
		// 古いレコードを削除
		$queryStr  = 'UPDATE _login_user ';
		$queryStr .=   'SET lu_deleted = true, ';	// 削除
		$queryStr .=     'lu_update_user_id = ?, ';
		$queryStr .=     'lu_update_dt = ? ';
		$queryStr .=   'WHERE lu_serial = ?';
		$this->execStatement($queryStr, array($userId, $now, $row['lu_serial']));

		// 新規レコード追加
		// パスワードはMD5変換して格納
		$queryStr  = 'INSERT INTO _login_user (';
		$queryStr .=   'lu_id, ';
		$queryStr .=   'lu_history_index, ';
		$queryStr .=   'lu_name, ';
		$queryStr .=   'lu_account, ';
		$queryStr .=   'lu_password, ';
		$queryStr .=   'lu_user_type, ';
		$queryStr .=   'lu_subject, ';
		$queryStr .=   'lu_widget_id, ';
		$queryStr .=   'lu_enable_login, ';
		$queryStr .=   'lu_create_user_id, ';
		$queryStr .=   'lu_create_dt ';
		$queryStr .= ') VALUES (';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		if ($isMd5){		// MD5化されているパスワードのときはそのまま格納
			$queryStr .=   '?, ';
		} else {
			$queryStr .=   'md5(?), ';
		}
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?) ';
		$this->execStatement($queryStr, array($row['lu_id'], $row['lu_history_index'] + 1, $row['lu_name'], $row['lu_account'], $password, $row['lu_user_type'], $row['lu_subject'], $row['lu_widget_id'], $row['lu_enable_login'], $userId, $now));
		
		// トランザクション確定
		if ($startTran) $ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ログインアカウントを変更
	 *
	 * @param int		$id			ユーザID
	 * @param string 	$account	アカウント
	 * @return bool					成功 = true, 失敗= false
	 */
	function updateLoginUserAccount($id, $account)
	{
		global $gEnvManager;
		
		$startTran = false;			// この関数でトランザクションを開始したかどうか
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
				
		// トランザクション開始
		if (!$this->isInTransaction()){
			$this->startTransaction();
			$startTran = true;
		}
		
		// 指定のシリアルNoのレコードが削除状態でないかチェック
		$queryStr  = 'select * from _login_user ';
		$queryStr .=   'WHERE lu_id = ? ';
		$queryStr .=    'and lu_deleted = false';
		$ret = $this->selectRecord($queryStr, array($id), $row);
		if (!$ret){		// 登録レコードがないとき
			if ($startTran) $this->endTransaction();
			return false;
		}
		
		// 古いレコードを削除
		$queryStr  = 'UPDATE _login_user ';
		$queryStr .=   'SET lu_deleted = true, ';	// 削除
		$queryStr .=     'lu_update_user_id = ?, ';
		$queryStr .=     'lu_update_dt = ? ';
		$queryStr .=   'WHERE lu_serial = ?';
		$this->execStatement($queryStr, array($userId, $now, $row['lu_serial']));

		// 新規レコード追加
		$queryStr  = 'INSERT INTO _login_user (';
		$queryStr .=   'lu_id, ';
		$queryStr .=   'lu_history_index, ';
		$queryStr .=   'lu_name, ';
		$queryStr .=   'lu_account, ';
		$queryStr .=   'lu_password, ';
		$queryStr .=   'lu_user_type, ';
		$queryStr .=   'lu_subject, ';
		$queryStr .=   'lu_widget_id, ';
		$queryStr .=   'lu_enable_login, ';
		$queryStr .=   'lu_create_user_id, ';
		$queryStr .=   'lu_create_dt ';
		$queryStr .= ') VALUES (';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?) ';
		$ret = $this->execStatement($queryStr, array($row['lu_id'], $row['lu_history_index'] + 1, $row['lu_name'], $account, $row['lu_password'], $row['lu_user_type'], $row['lu_subject'], $row['lu_widget_id'], $row['lu_enable_login'], $userId, $now));
		
		// トランザクション確定
		if ($startTran) $ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ユーザのタイプを一般ユーザに変更
	 *
	 * @param int $userId			ユーザID
	 * @return						true=成功、false=失敗
	 */
	function makeNormalLoginUser($userId)
	{
		global $gEnvManager;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$updateUserId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
			
		// トランザクション開始
		$this->startTransaction();
		
		// 指定のシリアルNoのレコードが削除状態でないかチェック
		$historyIndex = 0;		// 履歴番号
		$queryStr  = 'SELECT * FROM _login_user ';
		$queryStr .=   'WHERE lu_id = ? AND lu_deleted = false';
		$ret = $this->selectRecord($queryStr, array($userId), $row);
		if ($ret){		// 既に登録レコードがあるとき
			if ($row['lu_user_type'] >= UserInfo::USER_TYPE_NORMAL){		// 既に一般ユーザ以上のときは終了
				$this->endTransaction();
				return true;
			}
			$historyIndex = $row['lu_history_index'] + 1;
		} else {		// 存在しない場合は終了
			$this->endTransaction();
			return false;
		}
		$userType = UserInfo::USER_TYPE_NORMAL;		// 一般ユーザ
		
		// 古いレコードを削除
		$queryStr  = 'UPDATE _login_user ';
		$queryStr .=   'SET lu_deleted = true, ';	// 削除
		$queryStr .=     'lu_update_user_id = ?, ';
		$queryStr .=     'lu_update_dt = ? ';
		$queryStr .=   'WHERE lu_serial = ?';
		$this->execStatement($queryStr, array($updateUserId, $now, $row['lu_serial']));

		// 新規レコードを追加
		$queryStr  = 'INSERT INTO _login_user (';
		$queryStr .=   'lu_id, ';
		$queryStr .=   'lu_history_index, ';
		$queryStr .=   'lu_name, ';
		$queryStr .=   'lu_account, ';
		$queryStr .=   'lu_password, ';
		$queryStr .=   'lu_user_type, ';
		$queryStr .=   'lu_subject, ';
		$queryStr .=   'lu_enable_login, ';
		$queryStr .=   'lu_widget_id, ';
		$queryStr .=   'lu_create_user_id, ';
		$queryStr .=   'lu_create_dt ';
		$queryStr .= ') VALUES (';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?, ';
		$queryStr .=   '?) ';
		$ret = $this->execStatement($queryStr, array($userId, $historyIndex, $row['lu_name'], $row['lu_account'], $row['lu_password'],
								$userType, $row['lu_subject'], $row['lu_enable_login'], $row['lu_widget_id'], $updateUserId, $now));
								
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ログインログを更新
	 *
	 * @param int	$id				ユーザID
	 * @param int  $logSerial		アクセスログシリアル番号
	 * @return						true=成功、false=失敗
	 */
	function updateLoginLog($id, $logSerial)
	{
		// トランザクション開始
		$this->startTransaction();

		$queryStr  = 'SELECT * FROM _login_log ';
		$queryStr .=   'WHERE ll_user_id = ? ';	
		$ret = $this->selectRecord($queryStr, array($id), $row);
		if ($ret){
			$queryStr  = 'UPDATE _login_log ';
			$queryStr .=   'SET ll_login_count = ?, ';
			$queryStr .=     'll_access_log_serial = ?, ';
			$queryStr .=     'll_pre_login_dt = ?, ';
			$queryStr .=     'll_last_login_dt = now() ';
			$queryStr .=   'WHERE ll_user_id = ?';
			$this->execStatement($queryStr, array($row['ll_login_count'] +1, $logSerial, $row['ll_last_login_dt'], $id));			
		} else {
			$queryStr = 'INSERT INTO _login_log (';
			$queryStr .=  'll_user_id, ';
			$queryStr .=  'll_login_count, ';
			$queryStr .=  'll_access_log_serial, ';
			$queryStr .=  'll_last_login_dt ';
			$queryStr .=  ') VALUES (';
			$queryStr .=  '?, ?, ?, now()';
			$queryStr .=  ')';
			$this->execStatement($queryStr, array($id, 1, $logSerial));	
		}
				
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * エラーのログインログを追加
	 *
	 * @param string $account		入力アカウント
	 * @param string $ip			IPアドレス
	 * @param int  $logSerial		アクセスログシリアル番号
	 * @return						true=成功、false=失敗
	 */
	function addErrorLoginLog($account, $ip, $logSerial)
	{
		// トランザクション開始
		$this->startTransaction();

		$queryStr = 'INSERT INTO _login_err_log (';
		$queryStr .=  'le_account, ';
		$queryStr .=  'le_ip, ';
		$queryStr .=  'le_access_log_serial ';
		$queryStr .=  ') VALUES (';
		$queryStr .=  '?, ?, ?';
		$queryStr .=  ')';
		$this->execStatement($queryStr, array($account, $ip, $logSerial));
				
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * 管理者一時キーを登録
	 *
	 * @param string $key			登録キー
	 * @param string $ip			IPアドレス
	 * @return						true=成功、false=失敗
	 */
	function addAdminKey($key, $ip)
	{
		// トランザクション開始
		$this->startTransaction();

		$queryStr = 'INSERT INTO _admin_key (';
		$queryStr .=  'ak_id, ';
		$queryStr .=  'ak_ip, ';
		$queryStr .=  'ak_create_dt ';
		$queryStr .=  ') VALUES (';
		$queryStr .=  '?, ?, now()';
		$queryStr .=  ')';
		$this->execStatement($queryStr, array($key, $ip));
				
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * 有効な管理者一時キーかどうか
	 *
	 * @param string $key	アカウント
	 * @param string $ip	ユーザのアクセス元IP
	 * @return				true=有効、false=無効
	 */
	function isValidAdminKey($key, $ip)
	{
		$queryStr = 'SELECT * FROM _admin_key ';
		$queryStr .=  'WHERE ak_id = ? ';
		$queryStr .=    'AND ak_ip = ?';
		$ret = $this->selectRecord($queryStr, array($key, $ip), $row);
		if ($ret){	// 有効期間もチェック
			$lifeTime = self::ADMIN_KEY_LIFETIME;
			if (strtotime("-$lifeTime minutes") <= strtotime($row['ak_create_dt'])){
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	/**
	 * ウィジェットパラメータ更新
	 *
	 * @param string  $widgetId			ウィジェットID
	 * @param string  $serializedParam	シリアライズしたデータ
	 * @param int     $userId			更新者ユーザID
	 * @param string  $now				現在日時
	 * @return bool						true = 成功、false = 失敗
	 */
	function updateWidgetParam($widgetId, $serializedParam, $userId, $now = null)
	{
		// 更新日時が設定されていないときは設定
		if (empty($now)) $now = date("Y/m/d H:i:s");	// 現在日時
		
		// トランザクション開始
		$this->startTransaction();
		
		// 前レコードの削除状態チェック
		$historyIndex = 0;
		$queryStr = 'SELECT * FROM _widget_param ';
		$queryStr .=  'WHERE wp_id = ? ';
		$queryStr .=  'ORDER BY wp_history_index DESC ';
		$queryStr .=    'LIMIT 1';
		$ret = $this->selectRecord($queryStr, array($widgetId), $row);
		if ($ret){
			$historyIndex = $row['wp_history_index'] + 1;
		
			// レコードが削除されていない場合は削除
			if (!$row['wp_deleted']){
				// 古いレコードを削除
				$queryStr  = 'UPDATE _widget_param ';
				$queryStr .=   'SET wp_deleted = true, ';	// 削除
				$queryStr .=     'wp_update_user_id = ?, ';
				$queryStr .=     'wp_update_dt = ? ';
				$queryStr .=   'WHERE wp_serial = ?';
				$ret = $this->execStatement($queryStr, array($userId, $now, $row['wp_serial']));
				if (!$ret){
					$this->endTransaction();
					return false;
				}
			}
		}
		
		// データを追加
		$queryStr = 'INSERT INTO _widget_param ';
		$queryStr .=  '(wp_id, wp_history_index, wp_param, wp_create_user_id, wp_create_dt) ';
		$queryStr .=  'VALUES ';
		$queryStr .=  '(?, ?, ?, ?, ?)';
		$this->execStatement($queryStr, array($widgetId, $historyIndex, $serializedParam, $userId, $now));
			
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * ウィジェットパラメータ取得
	 *
	 * @param  string $widgetId	ウィジェットID
	 * @return string		シリアライズされたパラメータデータ
	 */
	function getWidgetParam($widgetId)
	{
		$sql = 'SELECT wp_param FROM _widget_param WHERE wp_id = ? AND wp_deleted = false';
		$params = array($widgetId); 
		$this->selectRecord($sql, $params, $row);
		return $row['wp_param'];
    }
	/**
	 * すべての追加クラス情報を取得
	 *
	 * @param array  $rows			レコード
	 * @return			true=1行以上取得、false=取得せず
	 */
	function getAllAddons(&$rows)
	{
		$queryStr = 'SELECT * FROM _addons WHERE ao_autoload = true ORDER BY ao_index';
		$retValue = $this->selectRecords($queryStr, array(), $rows);
		return $retValue;
	}
	/**
	 * メールフォームを取得
	 *
	 * @param  string $formId	フォームID
	 * @param  array  $row		取得レコード
	 * @return bool				true=正常取得、false=取得できず
	 */
	function getMailForm($formId, &$row)
	{
		$sql = 'SELECT * FROM _mail_form WHERE mf_id = ? AND mf_deleted = false';
		return $this->selectRecord($sql, array($formId), $row);
    }
	/**
	 * メール送信ログを追加
	 *
	 * @param int $type				メール送信タイプ(0=未設定、1=自動送信、2=手動送信)
	 * @param string $widgetId		送信を行ったウィジェットID
	 * @param string $toAddress		送信先メールアドレス
	 * @param string $fromAddress	送信元メールアドレス
	 * @param string $subject		件名
	 * @param string $body			メール本文
	 * @return bool					true=正常、false=異常
	 */
	function addMailLog($type, $widgetId, $toAddress, $fromAddress, $subject, $body)
	{	
		// トランザクション開始
		$this->startTransaction();
		
		$queryStr = 'INSERT INTO _mail_send_log (';
		$queryStr .=  'ms_type, ';
		$queryStr .=  'ms_widget_id, ';
		$queryStr .=  'ms_to, ';
		$queryStr .=  'ms_from, ';
		$queryStr .=  'ms_subject, ';
		$queryStr .=  'ms_body, ';
		$queryStr .=  'ms_dt ';
		$queryStr .=  ') VALUES (';
		$queryStr .=  '?, ?, ?, ?, ?, ?, now()';
		$queryStr .=  ')';
		$ret = $this->execStatement($queryStr, array($type, $widgetId, $toAddress, $fromAddress, $subject, $body));
		
		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * コンテンツのビューカウントを更新
	 *
	 * @param string    $typeId				コンテンツタイプ
	 * @param int     	$serial				シリアル番号
	 * @param string	$day				日にち
	 * @param int		$hour				時間
	 * @return bool							true=成功, false=失敗
	 */
	function updateViewCount($typeId, $serial, $day, $hour)
	{
		// 現在日時を取得
		if (empty($day)) $day = date("Y/m/d");		// 日
		if (empty($hour)) $hour = (int)date("H");		// 時間
		
		// トランザクション開始
		$this->startTransaction();
		
		// 既にレコードが作成されている場合はレコードを更新
		$queryStr  = 'SELECT * FROM _view_count ';
		$queryStr .=   'WHERE vc_type_id = ? ';			// データタイプはデフォルトコンテンツ
		$queryStr .=     'AND vc_content_serial = ? ';
		$queryStr .=     'AND vc_date = ? ';
		$queryStr .=     'AND vc_hour = ? ';
		$ret = $this->selectRecord($queryStr, array($typeId, $serial, $day, $hour), $row);
		if ($ret){
			$count = $row['vc_count'];
			$count++;		// カウント数を更新
			$queryStr = 'UPDATE _view_count ';
			$queryStr .=  'SET vc_count = ? ';
			$queryStr .=  'WHERE vc_serial = ? ';
			$this->execStatement($queryStr, array($count, $row['vc_serial']));
		} else {
			$queryStr = 'INSERT INTO _view_count ';
			$queryStr .=  '(vc_type_id, vc_count, vc_content_serial, vc_date, vc_hour) ';
			$queryStr .=  'VALUES ';
			$queryStr .=  '(?, 1, ?, ?, ?)';
			$this->execStatement($queryStr, array($typeId, $serial, $day, $hour));
		}
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
	/**
	 * エラーログ書き込み
	 *
	 * @param  string $type		メッセージタイプ(info=情報,warn=警告,error=通常エラー,fatal=致命的エラー)
	 * @param  string $method	呼び出し元クラスメソッド(通常は「__METHOD__」)
	 * @param  string $message	メッセージ
	 * @return bool				true=成功、false=失敗
	 */
	function writeErrorLog($type, $method, $message)
	{
		global $gEnvManager;
		
		$logSerial = $gEnvManager->getCurrentAccessLogSerial();	// 現在のアクセスログシリアル番号
		
		// トランザクション開始
		$this->startTransaction();
		
		$sql = "INSERT INTO _error_log (el_type, el_method, el_message, el_access_log_serial, el_dt) VALUES (?, ?, ?, ?, now())";
		$params = array($type, $method, $message, $logSerial);
		$this->execStatement($sql, $params);

		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * デバッグ出力書き込み
	 *
	 * @param  string $method	呼び出し元クラスメソッド(通常は「__METHOD__」)
	 * @param  string $message	メッセージ
	 * @return bool				true=成功、false=失敗
	 */
	function debugOut($method, $message)
	{
		global $gEnvManager;
		
		$logSerial = $gEnvManager->getCurrentAccessLogSerial();	// 現在のアクセスログシリアル番号
		$time = sprintf('%01.06f', microtime(true) - M3_MTIME);
		
		// トランザクション開始
		$this->startTransaction();
		
		$sql = "INSERT INTO _debug (db_method, db_message, db_access_log_serial, db_time, db_dt) VALUES (?, ?, ?, ?, now())";
		$params = array($method, $message, $logSerial, $time);
		$this->execStatement($sql, $params);

		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * ウィジェット実行ログ書き込み
	 *
	 * ウィジェットの実行状況(公開アクセス)をログに残す
	 *
	 * @param string $widgetId		実行ウィジェット
	 * @param int $type				実行タイプ(0=ページからの実行、1=単体実行)
	 * @param string $cmd			実行コマンド
	 * @param string $message		メッセージ
	 * @return bool					true=ログ出力完了、false=失敗
	 */
	function writeWidgetLog($widgetId, $type, $cmd = '', $message = '')
	{
		global $gEnvManager;
		
		// ログを残さないときは終了
		if (!$gEnvManager->getWidgetLog()) return false;
		
		$logSerial = $gEnvManager->getCurrentAccessLogSerial();	// 現在のアクセスログシリアル番号
		
		// トランザクション開始
		$this->startTransaction();
		
		$sql = "INSERT INTO _widget_log (wl_widget_id, wl_type, wl_cmd, wl_message, wl_access_log_serial, wl_dt) VALUES (?, ?, ?, ?, ?, now())";
		$params = array($widgetId, $type, $cmd, $message, $logSerial);
		$this->execStatement($sql, $params);

		// トランザクション終了
		return $this->endTransaction();
	}
	/**
	 * 汎用連想配列型パラメータを取得
	 *
	 * @param string	$key			キー
	 * @param string	$name			名前
	 * @return string					値、未設定の場合は空文字列
	 */
	function getKeyValue($key, &$name)
	{
		$retValue = '';
		$queryStr  = 'SELECT * FROM _key_value ';
		$queryStr .=   'WHERE kv_deleted = false ';	// 削除されていない
		$queryStr .=   'AND kv_id = ? ';
		$ret = $this->selectRecord($queryStr, array($key), $row);
		if ($ret){
			$retValue = $row['kv_value'];
			$name = $row['kv_name'];
		}
		return $retValue;
	}
	/**
	 * 汎用連想配列型パラメータのキーをシリアル番号で取得
	 *
	 * @param int		$serial			シリアル番号
	 * @return string					キー文字列、未設定の場合は空文字列
	 */
	function getKeyBySerial($serial)
	{
		$retValue = '';
		$queryStr  = 'SELECT * FROM _key_value ';
		$queryStr .=   'WHERE kv_deleted = false ';	// 削除されていない
		$queryStr .=   'AND kv_serial = ? ';
		$ret = $this->selectRecord($queryStr, array($serial), $row);
		if ($ret){
			$retValue = $row['kv_id'];
		}
		return $retValue;
	}
	/**
	 * 汎用連想配列型パラメータを更新
	 *
	 * @param string $key		キーとなる項目値
	 * @param string $value		値
	 * @param string $name		名前
	 * @param string $group		グループID
	 * @return					true = 正常、false=異常
	 */
	function updateKeyValue($key, $value, $name='', $group = '')
	{
		global $gEnvManager;
		
		$now = date("Y/m/d H:i:s");	// 現在日時
		$userId = $gEnvManager->getCurrentUserId();	// 現在のユーザ
				
		// トランザクション開始
		$this->startTransaction();
		
		// 前レコードの削除状態チェック
		$historyIndex = 0;
//		$name = '';
		$desc = '';
		$queryStr = 'SELECT * FROM _key_value ';
		$queryStr .=  'WHERE kv_id = ? ';
		$queryStr .=  'ORDER BY kv_history_index DESC ';
		$ret = $this->selectRecord($queryStr, array($key), $row);
		if ($ret){
			$historyIndex = $row['kv_history_index'] + 1;
//			$name = $row['kv_name'];
			$desc = $row['kv_description'];
			
			// レコードが削除されていない場合は削除
			if (!$row['kv_deleted']){
				// 古いレコードを削除
				$queryStr  = 'UPDATE _key_value ';
				$queryStr .=   'SET kv_deleted = true, ';	// 削除
				$queryStr .=     'kv_update_user_id = ?, ';
				$queryStr .=     'kv_update_dt = ? ';
				$queryStr .=   'WHERE kv_serial = ?';
				$ret = $this->execStatement($queryStr, array($userId, $now, $row['kv_serial']));
				if (!$ret){
					$this->endTransaction();
					return false;
				}
			}
		}
		
		// データを追加
		$queryStr = 'INSERT INTO _key_value ';
		$queryStr .=  '(kv_id, kv_history_index, kv_value, kv_name, kv_description, kv_group_id, kv_create_user_id, kv_create_dt) ';
		$queryStr .=  'VALUES ';
		$queryStr .=  '(?, ?, ?, ?, ?, ?, ?, ?)';
		$this->execStatement($queryStr, array($key, $historyIndex, $value, $name, $desc, $group, $userId, $now));
			
		// トランザクション確定
		$ret = $this->endTransaction();
		return $ret;
	}
}
?>
